@@ -1,5 +1,8 @@
 #!/bin/sh
 
+set -o xtrace
+set -o errexit
+
 #
 # build_snapshot_rpm.sh
 #
@@ -29,7 +32,7 @@ for arg in "$@"; do
     echo "  current repository."
     echo ""
     echo "  This script must be called from the base directory of the repository, and"
-    echo "  requires utilites from these packages: rpmdevtools, rpm-build, mock, wget"
+    echo "  requires utilites from these packages: rpm-build, mock, wget"
     echo ""
     exit
   fi
@@ -40,21 +43,11 @@ spec_file=../mongo-c-driver.spec
 spec_url=https://src.fedoraproject.org/rpms/mongo-c-driver/raw/master/f/mongo-c-driver.spec
 config=${MOCK_TARGET_CONFIG:=fedora-28-x86_64}
 
-if [ ! -x /usr/bin/rpmdev-bumpspec ]; then
-  echo "Missing the rpmdev-bumpspec utility from the rpmdevtools package"
-  exit 1
-fi
-
 if [ ! -x /usr/bin/rpmbuild -o ! -x /usr/bin/rpmspec ]; then
   echo "Missing the rpmbuild or rpmspec utility from the rpm-build package"
   exit 1
 fi
 
-if [ ! -f VERSION_CURRENT ]; then
-  echo "This script must be called from the base directory of the package"
-  exit 1
-fi
-
 if [ ! -d .git ]; then
   echo "This script only works from within a repository"
   exit 1
@@ -86,29 +79,49 @@ if [ "${package}" != "${changelog_package}" ]; then
   exit 1
 fi
 
+build_dir=$(basename $(pwd))
+
+sudo mock -r ${config} --bootstrap-chroot --old-chroot --clean
+sudo mock -r ${config} --bootstrap-chroot --old-chroot --init
+mock_root=$(sudo mock -r ${config} --bootstrap-chroot --old-chroot --print-root-path)
+sudo mock -r ${config} --bootstrap-chroot --old-chroot --install rpmdevtools git rpm-build cmake python GitPython python2-sphinx
+sudo mock -r ${config} --bootstrap-chroot --old-chroot --copyin "$(pwd)" "$(pwd)/${spec_file}" /tmp
+if [ ! -f VERSION_CURRENT ]; then
+  sudo mock -r ${config} --bootstrap-chroot --old-chroot --cwd "/tmp/${build_dir}" --chroot -- /bin/sh -c "(
+    set -o xtrace ;
+    python build/calc_release_version.py | sed -E 's/([^-]+).*/\1/' > VERSION_CURRENT ;
+    python build/calc_release_version.py -p > VERSION_RELEASED
+    )"
+  sudo mock -r ${config} --bootstrap-chroot --old-chroot --copyout "/tmp/${build_dir}/VERSION_CURRENT" "/tmp/${build_dir}/VERSION_RELEASED" .
+fi
+
 bare_upstream_version=$(sed -E 's/([^-]+).*/\1/' VERSION_CURRENT)
 # Upstream version in the .spec file cannot have hyphen (-); replace the current
 # version so that the dist tarball version does not have a pre-release component
 echo ${bare_upstream_version} > VERSION_CURRENT
 echo "Found bare upstream version: ${bare_upstream_version}"
 git_rev="$(git rev-parse --short HEAD)"
-snapshot_version="${bare_upstream_version}-0.$(date +%Y%m%d).git${git_rev}"
+snapshot_version="${bare_upstream_version}-0.$(date +%Y%m%d)+git${git_rev}"
 echo "Upstream snapshot version: ${snapshot_version}"
 current_package_version=$(rpmspec --srpm -q --qf "%{version}-%{release}" ${spec_file})
 
 if [ -n "${current_package_version##*${git_rev}*}" ]; then
   echo "Making RPM changelog entry"
-  rpmdev-bumpspec --comment="Built from Git Snapshot." --userstring="Test User <test@example.com>" --new="${snapshot_version}%{?dist}" ${spec_file}
+  sudo mock -r ${config} --bootstrap-chroot --old-chroot --cwd "/tmp/${build_dir}" --chroot -- rpmdev-bumpspec --comment="Built from Git Snapshot." --userstring="Test User <test@example.com>" --new="${snapshot_version}%{?dist}" ${spec_file}
 fi
 
-DIR=$(dirname $0)
-. $DIR/find-cmake.sh
+sudo mock -r ${config} --bootstrap-chroot --old-chroot --copyout "/tmp/${build_dir}/${spec_file}" ..
 
-mkdir cmake-build
-pushd cmake-build
-$CMAKE -DENABLE_MAN_PAGES=ON -DENABLE_HTML_DOCS=ON -DENABLE_ZLIB=BUNDLED -DENABLE_BSON=ON ..
-make -j 8 dist
-popd
+sudo mock -r ${config} --bootstrap-chroot --old-chroot --cwd "/tmp/${build_dir}" --chroot -- /bin/sh -c "(
+  set -o xtrace ;
+  [ -d cmake-build ] || mkdir cmake-build ;
+  cd cmake-build ;
+  /usr/bin/cmake -DENABLE_MAN_PAGES=ON -DENABLE_HTML_DOCS=ON -DENABLE_ZLIB=BUNDLED -DENABLE_BSON=ON .. ;
+  make -j 8 dist
+  )"
+
+[ -d cmake-build ] || mkdir cmake-build
+sudo mock -r ${config} --bootstrap-chroot --old-chroot --copyout "/tmp/${build_dir}/cmake-build/${package}*.tar.gz" cmake-build
 
 [ -d ~/rpmbuild/SOURCES ] || mkdir -p ~/rpmbuild/{BUILD,BUILDROOT,RPMS,SOURCES,SPECS,SRPMS}
 mv cmake-build/${package}*.tar.gz ~/rpmbuild/SOURCES/
@@ -116,5 +129,22 @@ mv cmake-build/${package}*.tar.gz ~/rpmbuild/SOURCES/
 echo "Building source RPM ..."
 rpmbuild -bs ${spec_file}
 echo "Building binary RPMs ..."
-sudo mock --rootdir="$(readlink -f ../mock-root)" --resultdir="$(readlink -f ../mock-result)" -r ${config} --rebuild ~/rpmbuild/SRPMS/${package}-${snapshot_version}*.src.rpm
+mock_result=$(readlink -f ../mock-result)
+sudo mock --resultdir="${mock_result}" --bootstrap-chroot --old-chroot -r ${config} --no-clean --no-cleanup-after --rebuild ~/rpmbuild/SRPMS/${package}-${snapshot_version}*.src.rpm
+sudo mock -r ${config} --bootstrap-chroot --old-chroot --copyin "${mock_result}" /tmp
+
+sudo mock -r ${config} --bootstrap-chroot --old-chroot --cwd "/tmp/${build_dir}" --chroot -- /bin/sh -c "(
+  set -o xtrace &&
+  rpm -Uvh ../mock-result/*.rpm &&
+  gcc -I/usr/include/libmongoc-1.0 -I/usr/include/libbson-1.0 -o example-client src/libmongoc/examples/example-client.c -lmongoc-1.0 -lbson-1.0
+  )"
+
+if [ ! -e "${mock_root}/tmp/${build_dir}/example-client" ]; then
+  echo "Example was not built!"
+  sudo mock -r ${config} --bootstrap-chroot --old-chroot --clean
+  exit 1
+fi
+
+sudo mock -r ${config} --bootstrap-chroot --old-chroot --clean
+(cd "${mock_result}" ; tar zcvf ../rpm.tar.gz *.rpm)
 
