@@ -22,6 +22,7 @@ set -o errexit  # Exit the script with error if any of the commands fail
 #       SNAPPY                  Build against bundled or external Snappy, or none
 #       SSL                     Build against OpenSSL or native or none
 #       SASL                    Build against SASL or not
+#       SRV                     Whether to enable SRV: ON or OFF
 #       ENABLE_SHM_COUNTERS     Build with SHM counters
 
 # Options for this script.
@@ -89,9 +90,16 @@ fi
 DEBUG_FLAGS="${DEBUG_AND_RELEASE_FLAGS} -DCMAKE_BUILD_TYPE=Debug"
 RELEASE_FLAGS="${DEBUG_AND_RELEASE_FLAGS} -DCMAKE_BUILD_TYPE=RelWithDebInfo"
 
-DIR=$(dirname $0)
+# Where are we, without relying on realpath or readlink?
+if [ "$(dirname $0)" = "." ]; then
+   DIR="$(pwd)"
+elif [ "$(dirname $0)" = ".." ]; then
+   DIR="$(dirname "$(pwd)")"
+else
+   DIR="$(cd "$(dirname "$0")"; pwd)"
+fi
+
 . $DIR/find-cmake.sh
-. $DIR/set-path.sh
 
 # --strip-components is an GNU tar extension. Check if the platform
 # has GNU tar installed as `gtar`, otherwise we assume to be on
@@ -116,6 +124,10 @@ if [ "$RELEASE" = "ON" ]; then
    cd build-dir
 fi
 
+if [ "$SRV" = "OFF" ]; then
+   CONFIGURE_FLAGS="$CONFIGURE_FLAGS -DENABLE_SRV=OFF"
+fi
+
 # UndefinedBehaviorSanitizer configuration
 export UBSAN_OPTIONS="print_stacktrace=1 abort_on_error=1"
 # AddressSanitizer configuration
@@ -179,21 +191,22 @@ pkg-config --modversion libssl || true
 if [ "$ANALYZE" = "ON" ]; then
    # Clang static analyzer, available on Ubuntu 16.04 images.
    # https://clang-analyzer.llvm.org/scan-build.html
-   scan-build $CMAKE $CONFIGURE_FLAGS
+   scan-build $CMAKE $CONFIGURE_FLAGS .
 
    # Put clang static analyzer results in scan/ and fail build if warnings found.
    SCAN_BUILD="scan-build -o scan --status-bugs"
 else
-   $CMAKE $CONFIGURE_FLAGS
+   $CMAKE $CONFIGURE_FLAGS .
 fi
 
-openssl version
+$SCAN_BUILD make -j8 all
+
+. $DIR/add-build-dirs-to-paths.sh
 if [ -n "$SSL_VERSION" ]; then
    openssl version | grep -q $SSL_VERSION
 fi
 # This should fail when using fips capable OpenSSL when fips mode is enabled
 openssl md5 README.rst || true
-$SCAN_BUILD make -j8 all
 
 ulimit -c unlimited || true
 
@@ -203,15 +216,27 @@ if [ "$SKIP_TESTS" = "ON" ]; then
    exit 0
 fi
 
-# Write stderr to error.log and to console.
-# TODO: valgrind
+if [ "$VALGRIND" = "ON" ]; then
+   # Defines "run_valgrind" shell function.
+   . $DIR/valgrind.sh
+else
+   # Define a no-op function.
+   run_valgrind ()
+   {
+      $@
+   }
+fi
+
+# Write stderr to error.log and to console. Turn off tracing to avoid spurious
+# log messages that CHECK_LOG considers failures.
 mkfifo pipe || true
 if [ -e pipe ]; then
+   set +o xtrace
    tee error.log < pipe &
-   ./src/libmongoc/test-libmongoc -d -F test-results.json 2>pipe
+   run_valgrind ./src/libmongoc/test-libmongoc -d -F test-results.json 2>pipe
    rm pipe
 else
-   ./src/libmongoc/test-libmongoc -d -F test-results.json
+   run_valgrind ./src/libmongoc/test-libmongoc -d -F test-results.json
 fi
 
 # Check if the error.log exists, and is more than 0 byte
