@@ -19,10 +19,16 @@ fi
 
 if [ "$BSON_ONLY" ]; then
   BUILD_DIR=$(pwd)/build-dir-bson
-  INSTALL_DIR=$(pwd)/install-dir-bson
+  INSTALL_PREFIX=$(pwd)/install-dir-bson
 else
   BUILD_DIR=$(pwd)/build-dir-mongoc
-  INSTALL_DIR=$(pwd)/install-dir-mongoc
+  INSTALL_PREFIX=$(pwd)/install-dir-mongoc
+fi
+
+if [ "$DESTDIR" ]; then
+   INSTALL_DIR="${DESTDIR}/${INSTALL_PREFIX}"
+else
+   INSTALL_DIR=$INSTALL_PREFIX
 fi
 
 rm -rf $BUILD_DIR
@@ -41,14 +47,18 @@ else
 fi
 
 
-$CMAKE -DCMAKE_INSTALL_PREFIX=$INSTALL_DIR -DCMAKE_PREFIX_PATH=$INSTALL_DIR/lib/cmake $BSON_ONLY_OPTION .
+$CMAKE -DCMAKE_INSTALL_PREFIX=$INSTALL_PREFIX -DCMAKE_PREFIX_PATH=$INSTALL_DIR/lib/cmake $BSON_ONLY_OPTION .
 make
-make install
+if [ "$DESTDIR" ]; then
+   make DESTDIR=$DESTDIR install
+else
+   make install
+fi
 touch $INSTALL_DIR/lib/canary.txt
 
 ls -l $INSTALL_DIR/share/mongo-c-driver
 
-$INSTALL_DIR/share/mongo-c-driver/uninstall.sh
+make uninstall
 
 set +o xtrace
 
@@ -140,6 +150,12 @@ if [ -z "$BSON_ONLY" ]; then
     echo "$INSTALL_DIR/include/libmongoc-1.0 check ok"
   fi
 fi
+if test -f $INSTALL_DIR/share/mongo-c-driver/uninstall-bson.sh; then
+  echo "uninstall-bson.sh found!"
+  exit 1
+else
+  echo "uninstall-bson.sh check ok"
+fi
 if test -f $INSTALL_DIR/share/mongo-c-driver/uninstall.sh; then
   echo "uninstall.sh found!"
   exit 1
