@@ -1,10 +1,11 @@
 /* required on old Windows for rand_s to be defined */
 #define _CRT_RAND_S
 
-#include <bson.h>
+#include <bson/bson.h>
 #include <math.h>
 
 #include "TestSuite.h"
+#include "test-conveniences.h"
 
 static ssize_t
 test_bson_json_read_cb_helper (void *string, uint8_t *buf, size_t len)
@@ -905,106 +906,20 @@ _test_bson_json_read_compare (const char *json, int size, ...)
 static void
 test_bson_json_read (void)
 {
-   const char *json = "{ \n\
-      \"foo\" : \"bar\", \n\
-      \"bar\" : 12341, \n\
-      \"baz\" : 123.456, \n\
-      \"map\" : { \"a\" : 1 }, \n\
-      \"array\" : [ 1, 2, 3, 4 ], \n\
-      \"null\" : null, \n\
-      \"boolean\" : true, \n\
-      \"oid\" : { \n\
-        \"$oid\" : \"000000000000000000000000\" \n\
-      }, \n\
-      \"binary\" : { \n\
-        \"$type\" : \"00\", \n\
-        \"$binary\" : \"ZGVhZGJlZWY=\" \n\
-      }, \n\
-      \"regex\" : { \n\
-        \"$regularExpression\" : { \n\
-          \"pattern\": \"foo|bar\", \"options\" : \"ism\" \n\
-        } \n\
-      }, \n\
-      \"date\" : { \n\
-        \"$date\" : \"1970-01-01T00:00:10Z\" \n\
-      }, \n\
-      \"ref\" : { \n\
-        \"$ref\" : \"foo\", \n\
-        \"$id\" : {\"$oid\": \"000000000000000000000000\"} \n\
-      }, \n\
-      \"undefined\" : { \n\
-        \"$undefined\" : true \n\
-      }, \n\
-      \"minkey\" : { \n\
-        \"$minKey\" : 1 \n\
-      }, \n\
-      \"maxkey\" : { \n\
-        \"$maxKey\" : 1 \n\
-      }, \n\
-      \"timestamp\" : { \n\
-        \"$timestamp\" : { \n\
-           \"t\" : 100, \n\
-           \"i\" : 1000 \n\
-        } \n\
-      } \n\
-   } { \"after\": \"b\" } { \"twice\" : true }";
+   char *json = bson_strdup_printf (
+      "%s { \"after\": \"b\" } { \"twice\" : true }", json_with_all_types ());
 
    bson_oid_t oid;
    bson_t *first, *second, *third;
 
    bson_oid_init_from_string (&oid, "000000000000000000000000");
 
-   first =
-      BCON_NEW ("foo",
-                "bar",
-                "bar",
-                BCON_INT32 (12341),
-                "baz",
-                BCON_DOUBLE (123.456),
-                "map",
-                "{",
-                "a",
-                BCON_INT32 (1),
-                "}",
-                "array",
-                "[",
-                BCON_INT32 (1),
-                BCON_INT32 (2),
-                BCON_INT32 (3),
-                BCON_INT32 (4),
-                "]",
-                "null",
-                BCON_NULL,
-                "boolean",
-                BCON_BOOL (true),
-                "oid",
-                BCON_OID (&oid),
-                "binary",
-                BCON_BIN (BSON_SUBTYPE_BINARY, (const uint8_t *) "deadbeef", 8),
-                "regex",
-                BCON_REGEX ("foo|bar", "ism"),
-                "date",
-                BCON_DATE_TIME (10000),
-                "ref",
-                "{",
-                "$ref",
-                BCON_UTF8 ("foo"),
-                "$id",
-                BCON_OID (&oid),
-                "}",
-                "undefined",
-                BCON_UNDEFINED,
-                "minkey",
-                BCON_MINKEY,
-                "maxkey",
-                BCON_MAXKEY,
-                "timestamp",
-                BCON_TIMESTAMP (100, 1000));
-
+   first = bson_copy (bson_with_all_types ());
    second = BCON_NEW ("after", "b");
    third = BCON_NEW ("twice", BCON_BOOL (true));
 
    _test_bson_json_read_compare (json, 5, first, second, third, NULL);
+   bson_free (json);
 }
 
 static void
@@ -1037,6 +952,8 @@ test_bson_json_read_corrupt_utf8 (void)
                           BSON_JSON_ERROR_READ_CORRUPT_JS,
                           "invalid bytes in UTF8 string");
 
+   memset (&error, 0, sizeof error);
+
    BSON_ASSERT (!bson_new_from_json ((uint8_t *) bad_value, -1, &error));
    ASSERT_ERROR_CONTAINS (error,
                           BSON_ERROR_JSON,
@@ -1065,9 +982,11 @@ test_bson_json_read_corrupt_document (void)
                             "\x00";                    /* terminator */
 
    bson_t bson;
-   bson_init_static (&bson, (uint8_t *) bad_doc, sizeof (bad_doc));
+   BSON_ASSERT (
+      bson_init_static (&bson, (uint8_t *) bad_doc, sizeof (bad_doc)));
    BSON_ASSERT (!bson_as_json (&bson, NULL));
-   bson_init_static (&bson, (uint8_t *) bad_array, sizeof (bad_array));
+   BSON_ASSERT (
+      bson_init_static (&bson, (uint8_t *) bad_array, sizeof (bad_array)));
    BSON_ASSERT (!bson_as_json (&bson, NULL));
 }
 
@@ -1146,6 +1065,8 @@ test_bson_json_read_legacy_regex (void)
                           BSON_JSON_ERROR_READ_INVALID_PARAM,
                           "Missing \"$options\" after \"$regex\"");
 
+   memset (&error, 0, sizeof error);
+
    r = bson_init_from_json (&b, "{\"a\": {\"$options\": \"ix\"}}", -1, &error);
    BSON_ASSERT (!r);
    ASSERT_ERROR_CONTAINS (error,
@@ -1228,6 +1149,8 @@ test_bson_json_read_binary (void)
                           BSON_JSON_ERROR_READ_INVALID_PARAM,
                           "Missing \"base64\" after \"subType\"");
 
+   memset (&error, 0, sizeof error);
+
    /* no subType */
    r = bson_init_from_json (
       &b, "{\"b\": {\"$binary\": {\"base64\": \"Zm9v\"}}}", -1, &error);
@@ -1512,6 +1435,8 @@ test_bson_json_number_long (void)
                           BSON_JSON_ERROR_READ_INVALID_PARAM,
                           "Number \"9223372036854775808\" is out of range");
 
+   memset (&error, 0, sizeof error);
+
    /* INT64_MIN - 1 */
    r = bson_init_from_json (
       &b, "{\"x\": {\"$numberLong\": \"-9223372036854775809\"}}", -1, &error);
@@ -1522,6 +1447,8 @@ test_bson_json_number_long (void)
                           BSON_JSON_ERROR_READ_INVALID_PARAM,
                           "Number \"-9223372036854775809\" is out of range");
 
+   memset (&error, 0, sizeof error);
+
    r = bson_init_from_json (
       &b, "{\"x\": {\"$numberLong\": \"10000000000000000000\"}}", -1, &error);
 
@@ -1531,6 +1458,8 @@ test_bson_json_number_long (void)
                           BSON_JSON_ERROR_READ_INVALID_PARAM,
                           "Number \"10000000000000000000\" is out of range");
 
+   memset (&error, 0, sizeof error);
+
    /* INT64_MIN - 2 */
    r = bson_init_from_json (&b, "{\"x\": -10000000000000000000}", -1, &error);
 
@@ -2033,6 +1962,8 @@ test_bson_json_int64 (void)
                           BSON_JSON_ERROR_READ_INVALID_PARAM,
                           "Number \"9223372036854775808\" is out of range");
 
+   memset (&error, 0, sizeof error);
+
    /* INT64_MIN - 1 */
    BSON_ASSERT (
       !bson_init_from_json (&b, "{ \"x\": -9223372036854775809 }", -1, &error));
@@ -2041,6 +1972,8 @@ test_bson_json_int64 (void)
                           BSON_JSON_ERROR_READ_INVALID_PARAM,
                           "Number \"-9223372036854775809\" is out of range");
 
+   memset (&error, 0, sizeof error);
+
    BSON_ASSERT (
       !bson_init_from_json (&b, "{ \"x\": 10000000000000000000 }", -1, &error));
    ASSERT_ERROR_CONTAINS (error,
@@ -2048,6 +1981,8 @@ test_bson_json_int64 (void)
                           BSON_JSON_ERROR_READ_INVALID_PARAM,
                           "Number \"10000000000000000000\" is out of range");
 
+   memset (&error, 0, sizeof error);
+
    /* INT64_MIN - 2 */
    BSON_ASSERT (!bson_init_from_json (
       &b, "{ \"x\": -10000000000000000000 }", -1, &error));
@@ -2132,6 +2067,7 @@ test_bson_json_double_overflow (void)
                              "out of range");
 
       bson_free (j);
+      memset (&error, 0, sizeof error);
 
       /* same test with canonical Extended JSON */
       j = bson_strdup_printf ("{ \"d\" : { \"$numberDouble\" : \"%s\" } }", *p);
@@ -2186,6 +2122,7 @@ test_bson_json_nan (void)
                              BSON_ERROR_JSON,
                              BSON_JSON_ERROR_READ_CORRUPT_JS,
                              "Got parse error at");
+      memset (&error, 0, sizeof error);
    }
 
    for (j = partial; *j; j++) {
@@ -2194,6 +2131,7 @@ test_bson_json_nan (void)
                              BSON_ERROR_JSON,
                              BSON_JSON_ERROR_READ_CORRUPT_JS,
                              "Incomplete JSON");
+      memset (&error, 0, sizeof error);
    }
 }
 
@@ -2251,6 +2189,7 @@ test_bson_json_infinity (void)
                              BSON_ERROR_JSON,
                              BSON_JSON_ERROR_READ_CORRUPT_JS,
                              "Got parse error at");
+      memset (&error, 0, sizeof error);
    }
 
    for (j = partial; *j; j++) {
@@ -2259,6 +2198,7 @@ test_bson_json_infinity (void)
                              BSON_ERROR_JSON,
                              BSON_JSON_ERROR_READ_CORRUPT_JS,
                              "Incomplete JSON");
+      memset (&error, 0, sizeof error);
    }
 }
 
@@ -2676,6 +2616,12 @@ test_bson_array_as_json (void)
    BSON_ASSERT (len == 5);
    bson_free (str);
 
+   /* test corrupted bson */
+   BSON_APPEND_UTF8 (&d, "1", "\x80");  /* bad UTF-8 */
+   str = bson_array_as_json (&d, &len);
+   BSON_ASSERT (!str);
+   BSON_ASSERT (!len);
+
    bson_destroy (&d);
 }
 
@@ -2756,6 +2702,7 @@ test_bson_json_null_in_str (void)
       (const uint8_t *) bad_json, sizeof (bad_json) - 1, &err));
    ASSERT_ERROR_CONTAINS (
       err, BSON_ERROR_JSON, BSON_JSON_ERROR_READ_CORRUPT_JS, "Got parse error");
+   memset (&err, 0, sizeof err);
    ASSERT (!bson_new_from_json (
       (const uint8_t *) cdriver2305, sizeof (cdriver2305) - 1, &err));
    ASSERT_ERROR_CONTAINS (
