@@ -13,7 +13,6 @@ extensions = [
     'mongoc',
     'taglist',
     'sphinx.ext.intersphinx',
-    'sphinx.ext.extlinks',
 ]
 
 # General information about the project.
@@ -24,23 +23,8 @@ author = 'MongoDB, Inc'
 version_path = os.path.join(
     os.path.dirname(__file__), '../../..', 'VERSION_CURRENT')
 version = open(version_path).read().strip()
-release_path = os.path.join(
-    os.path.dirname(__file__), '../../..', 'VERSION_RELEASED')
-release = open(release_path).read().strip()
-release_major, release_minor, release_patch = release.split('.')
-release_download = 'https://github.com/mongodb/mongo-c-driver/releases/download/{0}/mongo-c-driver-{0}.tar.gz'.format(release)
-rst_prolog = """
-.. |release_major| replace:: %(release_major)s
-
-.. |release_minor| replace:: %(release_minor)s
-
-.. |release_patch| replace:: %(release_patch)s
-
-.. |release_download| replace:: https://github.com/mongodb/mongo-c-driver/releases/download/%(release)s/mongo-c-driver-%(release)s.tar.gz
-""" % locals()
 
 # The extension requires the "base" to contain '%s' exactly once, but we never intend to use it though
-extlinks = {'release': (release_download+'%s', '')}
 
 language = 'en'
 exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']
