@@ -24,7 +24,6 @@ const char *COLLECTION_NAME = "things";
 #include "../doc-common-insert.c"
 #include "explain.c"
 #include "copydb.c"
-#include "clone-collection.c"
 
 
 int
@@ -104,12 +103,6 @@ main (int argc, char *argv[])
          res = EXIT_FAILURE;
          goto cleanup;
       }
-
-      printf ("clone collection\n");
-      if (!clone_collection (database, other_host_and_port)) {
-         res = EXIT_FAILURE;
-         goto cleanup;
-      }
    }
 
 cleanup:
