@@ -14,30 +14,28 @@
  * limitations under the License.
  */
 
+#include "mongoc/mongoc-prelude.h"
+
 #ifndef MONGOC_CLUSTER_PRIVATE_H
 #define MONGOC_CLUSTER_PRIVATE_H
 
-#if !defined(MONGOC_COMPILATION)
-#error "Only <mongoc.h> can be included directly."
-#endif
+#include <bson/bson.h>
 
-#include <bson.h>
-
-#include "mongoc-array-private.h"
-#include "mongoc-buffer-private.h"
-#include "mongoc-config.h"
-#include "mongoc-client.h"
-#include "mongoc-list-private.h"
-#include "mongoc-opcode.h"
-#include "mongoc-rpc-private.h"
-#include "mongoc-server-stream-private.h"
-#include "mongoc-set-private.h"
-#include "mongoc-stream.h"
-#include "mongoc-topology-private.h"
-#include "mongoc-topology-description-private.h"
-#include "mongoc-write-concern.h"
-#include "mongoc-scram-private.h"
-#include "mongoc-cmd-private.h"
+#include "mongoc/mongoc-array-private.h"
+#include "mongoc/mongoc-buffer-private.h"
+#include "mongoc/mongoc-config.h"
+#include "mongoc/mongoc-client.h"
+#include "mongoc/mongoc-list-private.h"
+#include "mongoc/mongoc-opcode.h"
+#include "mongoc/mongoc-rpc-private.h"
+#include "mongoc/mongoc-server-stream-private.h"
+#include "mongoc/mongoc-set-private.h"
+#include "mongoc/mongoc-stream.h"
+#include "mongoc/mongoc-topology-private.h"
+#include "mongoc/mongoc-topology-description-private.h"
+#include "mongoc/mongoc-write-concern.h"
+#include "mongoc/mongoc-scram-private.h"
+#include "mongoc/mongoc-cmd-private.h"
 
 BSON_BEGIN_DECLS
 
@@ -80,6 +78,9 @@ mongoc_cluster_init (mongoc_cluster_t *cluster,
 void
 mongoc_cluster_destroy (mongoc_cluster_t *cluster);
 
+void
+mongoc_cluster_disconnect (mongoc_cluster_t *cluster);
+
 void
 mongoc_cluster_disconnect_node (mongoc_cluster_t *cluster,
                                 uint32_t id,
