@@ -14,13 +14,11 @@
  * limitations under the License.
  */
 
+#include "mongoc/mongoc-prelude.h"
+
 #ifndef MONGOC_UTIL_PRIVATE_H
 #define MONGOC_UTIL_PRIVATE_H
 
-#if !defined(MONGOC_COMPILATION)
-#error "Only <mongoc/mongoc.h> can be included directly."
-#endif
-
 #include <bson/bson.h>
 #include "mongoc/mongoc.h"
 
@@ -50,8 +48,7 @@
 #endif
 
 #ifndef _WIN32
-#define MONGOC_PRINTF_FORMAT(a, b) \
-   __attribute__ ((format (__printf__, a, b)))
+#define MONGOC_PRINTF_FORMAT(a, b) __attribute__ ((format (__printf__, a, b)))
 #else
 #define MONGOC_PRINTF_FORMAT(a, b) /* no-op */
 #endif
