@@ -70,7 +70,6 @@ json_test_ctx_init (json_test_ctx_t *ctx,
    ctx->n_events = 0;
    bson_init (&ctx->events);
    ctx->test_framework_uri = test_framework_get_uri ();
-   ctx->cursor_id = 0;
    ctx->acknowledged = true;
    ctx->verbose = test_framework_getenv_bool ("MONGOC_TEST_MONITORING_VERBOSE");
    bson_init (&ctx->lsids[0]);
@@ -225,6 +224,10 @@ convert_bulk_write_result (const bson_t *doc, bson_t *r)
 
             bson_append_array_end (r, &upserted);
          }
+      } else if (BSON_ITER_IS_KEY (&iter, "insertedIds")) {
+         /* tests expect insertedIds, but they're optional and we omit them */
+      } else {
+         BSON_APPEND_VALUE (r, bson_iter_key (&iter), bson_iter_value (&iter));
       }
    }
 }
@@ -279,9 +282,7 @@ convert_spec_result (const char *op_name,
 
 
 static bool
-get_successful_result (const bson_t *test,
-                       const bson_t *operation,
-                       bson_value_t *value)
+get_result (const bson_t *test, const bson_t *operation, bson_value_t *value)
 {
    const char *op_name;
    bson_value_t pre_conversion;
@@ -365,11 +366,11 @@ check_error_code_name (const bson_t *operation, const bson_error_t *error)
 {
    const char *code_name;
 
-   if (!bson_has_field (operation, "result.errorCodeName")) {
+   if (!bson_has_field (operation, "errorCodeName")) {
       return;
    }
 
-   code_name = bson_lookup_utf8 (operation, "result.errorCodeName");
+   code_name = bson_lookup_utf8 (operation, "errorCodeName");
    ASSERT_CMPUINT32 (error->code, ==, error_code_from_name (code_name));
 }
 
@@ -379,11 +380,11 @@ check_error_contains (const bson_t *operation, const bson_error_t *error)
 {
    const char *msg;
 
-   if (!bson_has_field (operation, "result.errorContains")) {
+   if (!bson_has_field (operation, "errorContains")) {
       return;
    }
 
-   msg = bson_lookup_utf8 (operation, "result.errorContains");
+   msg = bson_lookup_utf8 (operation, "errorContains");
    ASSERT_CONTAINS (error->message, msg);
 }
 
@@ -397,13 +398,13 @@ check_error_labels_contain (const bson_t *operation, const bson_value_t *result)
    bson_iter_t expected_label;
    const char *expected_label_str;
 
-   if (!bson_has_field (operation, "result.errorLabelsContain")) {
+   if (!bson_has_field (operation, "errorLabelsContain")) {
       return;
    }
 
    BSON_ASSERT (bson_iter_init (&operation_iter, operation));
    BSON_ASSERT (bson_iter_find_descendant (
-      &operation_iter, "result.errorLabelsContain", &expected_labels));
+      &operation_iter, "errorLabelsContain", &expected_labels));
    BSON_ASSERT (bson_iter_recurse (&expected_labels, &expected_label));
 
    /* if the test has "errorLabelsContain" then result must be an error reply */
@@ -428,7 +429,7 @@ check_error_labels_omit (const bson_t *operation, const bson_value_t *result)
    bson_t omitted_labels;
    bson_iter_t omitted_label;
 
-   if (!bson_has_field (operation, "result.errorLabelsOmit")) {
+   if (!bson_has_field (operation, "errorLabelsOmit")) {
       return;
    }
 
@@ -442,7 +443,7 @@ check_error_labels_omit (const bson_t *operation, const bson_value_t *result)
       return;
    }
 
-   bson_lookup_doc (operation, "result.errorLabelsOmit", &omitted_labels);
+   bson_lookup_doc (operation, "errorLabelsOmit", &omitted_labels);
    BSON_ASSERT (bson_iter_init (&omitted_label, &omitted_labels));
    while (bson_iter_next (&omitted_label)) {
       if (mongoc_error_has_label (&reply,
@@ -492,74 +493,67 @@ check_result (const bson_t *test,
               const bson_value_t *result,
               const bson_error_t *error)
 {
-   bson_value_t expected_result;
-   char errmsg[1000];
-   match_ctx_t ctx = {0};
+   bson_t expected_doc;
+   bson_value_t expected_value;
+   match_ctx_t ctx = {{0}};
 
-   ctx.errmsg = errmsg;
-   ctx.errmsg_len = sizeof errmsg;
+   if (!get_result (test, operation, &expected_value)) {
+      /* if there's no "result", e.g. in the command monitoring tests,
+       * we don't know if the command is expected to succeed or fail */
+      return;
+   }
 
-   /* retryable writes tests specify error: false at the end of the whole test:
+   if (expected_value.value_type == BSON_TYPE_DOCUMENT) {
+      bson_init_from_value (&expected_doc, &expected_value);
+      if (bson_has_field (&expected_doc, "errorCodeName") ||
+          bson_has_field (&expected_doc, "errorContains") ||
+          bson_has_field (&expected_doc, "errorLabelsContain") ||
+          bson_has_field (&expected_doc, "errorLabelsOmit")) {
+         /* Expect the operation has failed. Transactions tests specify errors
+          * per-operation, with one or more details:
+          *    operations:
+          *      - name: insertOne
+          *        arguments: ...
+          *        result:
+          *          errorCodeName: WriteConflict
+          *          errorContains: "message substring"
+          *          errorLabelsContain: ["TransientTransactionError"]
+          *          errorLabelsOmit: ["UnknownTransactionCommitResult"]
+          */
+
+         check_success_expected (&expected_doc, succeeded, false, error);
+         check_error_code_name (&expected_doc, error);
+         check_error_contains (&expected_doc, error);
+         check_error_labels_contain (&expected_doc, result);
+         check_error_labels_omit (&expected_doc, result);
+         bson_value_destroy (&expected_value);
+         return;
+      }
+   }
+
+   /* retryable writes tests and CRUD tests specify error: <bool> at the end of
+    * the whole test:
     *   operation:
     *     name: insertOne
     *   outcome:
     *     error: true
     */
-   if (bson_has_field (test, "outcome.result.error")) {
-      check_success_expected (
-         operation,
-         succeeded,
-         _mongoc_lookup_bool (test, "outcome.result.error", false),
-         error);
+   check_success_expected (operation,
+                           succeeded,
+                           !_mongoc_lookup_bool (test, "outcome.error", false),
+                           error);
+
+   BSON_ASSERT (result);
+   if (!match_bson_value (result, &expected_value, &ctx)) {
+      test_error ("Error in \"%s\" test %s\n"
+                  "Expected:\n%s\nActual:\n%s",
+                  bson_lookup_utf8 (test, "description"),
+                  ctx.errmsg,
+                  value_to_str (&expected_value),
+                  value_to_str (result));
    }
 
-   /* if there's no "result", e.g. in the command monitoring tests, we don't
-    * know if the command is expected to succeed or fail */
-   if (!bson_has_field (operation, "result")) {
-      return;
-   }
-
-   if (!bson_has_field (operation, "result.errorCodeName") &&
-       !bson_has_field (operation, "result.errorContains") &&
-       !bson_has_field (operation, "result.errorLabelsContain") &&
-       !bson_has_field (operation, "result.errorLabelsOmit")) {
-      /* expect the operation has succeeded */
-      check_success_expected (operation, succeeded, true, error);
-      if (!get_successful_result (test, operation, &expected_result)) {
-         /* some tests don't verify the return value */
-         return;
-      }
-
-      BSON_ASSERT (result);
-      if (!match_bson_value (result, &expected_result, &ctx)) {
-         test_error ("Error in \"%s\" test %s\n"
-                     "Expected:\n%s\nActual:\n%s",
-                     bson_lookup_utf8 (test, "description"),
-                     ctx.errmsg,
-                     value_to_str (&expected_result),
-                     value_to_str (result));
-      }
-
-      bson_value_destroy (&expected_result);
-      return;
-   }
-
-   /* transactions tests specify errors per-operation, with one or more details:
-    *    operations:
-    *      - name: insertOne
-    *        arguments: ...
-    *        result:
-    *          errorCodeName: WriteConflict
-    *          errorContains: "message substring"
-    *          errorLabelsContain: ["TransientTransactionError"]
-    *          errorLabelsOmit: ["UnknownTransactionCommitResult"]
-    */
-
-   check_success_expected (operation, succeeded, false, error);
-   check_error_code_name (operation, error);
-   check_error_contains (operation, error);
-   check_error_labels_contain (operation, result);
-   check_error_labels_omit (operation, result);
+   bson_value_destroy (&expected_value);
 }
 
 
@@ -583,6 +577,12 @@ add_request_to_bulk (mongoc_bulk_operation_t *bulk,
       bson_value_destroy (&array_filters);
    }
 
+   if (bson_has_field (&args, "collation")) {
+      bson_t collation;
+      bson_lookup_doc (&args, "collation", &collation);
+      BSON_APPEND_DOCUMENT (&opts, "collation", &collation);
+   }
+
    if (!strcmp (name, "deleteMany")) {
       bson_t filter;
 
@@ -762,19 +762,19 @@ single_write (mongoc_collection_t *collection,
    if (!strcmp (name, "deleteMany")) {
       bson_t filter;
       bson_lookup_doc (&args, "filter", &filter);
-      COPY_EXCEPT (&args, &opts, "filter");
+      COPY_EXCEPT ("filter");
       r = mongoc_collection_delete_many (
          collection, &filter, &opts, &reply, &error);
    } else if (!strcmp (name, "deleteOne")) {
       bson_t filter;
       bson_lookup_doc (&args, "filter", &filter);
-      COPY_EXCEPT (&args, &opts, "filter");
+      COPY_EXCEPT ("filter");
       r = mongoc_collection_delete_one (
          collection, &filter, &opts, &reply, &error);
    } else if (!strcmp (name, "insertOne")) {
       bson_t document;
       bson_lookup_doc (&args, "document", &document);
-      COPY_EXCEPT (&args, &opts, "document");
+      COPY_EXCEPT ("document");
       r = mongoc_collection_insert_one (
          collection, &document, &opts, &reply, &error);
    } else if (!strcmp (name, "replaceOne")) {
@@ -782,7 +782,7 @@ single_write (mongoc_collection_t *collection,
       bson_t replacement;
       bson_lookup_doc (&args, "filter", &filter);
       bson_lookup_doc (&args, "replacement", &replacement);
-      COPY_EXCEPT (&args, &opts, "filter", "replacement");
+      COPY_EXCEPT ("filter", "replacement");
       r = mongoc_collection_replace_one (
          collection, &filter, &replacement, &opts, &reply, &error);
    } else if (!strcmp (name, "updateMany")) {
@@ -790,7 +790,7 @@ single_write (mongoc_collection_t *collection,
       bson_t update;
       bson_lookup_doc (&args, "filter", &filter);
       bson_lookup_doc (&args, "update", &update);
-      COPY_EXCEPT (&args, &opts, "filter", "update");
+      COPY_EXCEPT ("filter", "update");
       r = mongoc_collection_update_many (
          collection, &filter, &update, &opts, &reply, &error);
    } else if (!strcmp (name, "updateOne")) {
@@ -798,7 +798,7 @@ single_write (mongoc_collection_t *collection,
       bson_t update;
       bson_lookup_doc (&args, "filter", &filter);
       bson_lookup_doc (&args, "update", &update);
-      COPY_EXCEPT (&args, &opts, "filter", "update");
+      COPY_EXCEPT ("filter", "update");
       r = mongoc_collection_update_one (
          collection, &filter, &update, &opts, &reply, &error);
    } else {
@@ -988,11 +988,26 @@ insert_many (mongoc_collection_t *collection,
       "ordered",
       _mongoc_lookup_bool (&args, "options.ordered", true /* default */));
 
-   COPY_EXCEPT (&args, &opts, "documents", "options");
+   COPY_EXCEPT ("documents", "options");
 
    r = mongoc_collection_insert_many (
       collection, (const bson_t **) doc_ptrs, n, &opts, &reply, &error);
 
+   /* CRUD tests may specify a write result even if an error is expected.
+    * From the CRUD spec test readme:
+    * "drivers may choose to check the result object if their BulkWriteException
+    * (or equivalent) provides access to a write result object"
+    * The C driver does not return a full write result for an "InsertMany" on
+    * error, but instead just the insertedCount.
+    */
+   if (!r) {
+      BSON_APPEND_INT64 (&reply, "deletedCount", 0);
+      BSON_APPEND_INT64 (&reply, "matchedCount", 0);
+      BSON_APPEND_INT64 (&reply, "modifiedCount", 0);
+      BSON_APPEND_INT64 (&reply, "upsertedCount", 0);
+      BSON_APPEND_DOCUMENT (&reply, "upsertedIds", tmp_bson ("{}"));
+   }
+
    value_init_from_doc (&value, &reply);
    check_result (test, operation, r, &value, &error);
    bson_value_destroy (&value);
@@ -1005,6 +1020,51 @@ insert_many (mongoc_collection_t *collection,
 }
 
 
+static void
+rename_op (mongoc_collection_t *collection,
+           const bson_t *test,
+           const bson_t *operation,
+           mongoc_client_session_t *session,
+           mongoc_write_concern_t *wc)
+{
+   bson_t args;
+   const char *db;
+   const char *to;
+   bson_error_t error;
+   bool res;
+
+   bson_lookup_doc (operation, "arguments", &args);
+   db = bson_lookup_utf8 (operation, "database");
+   to = bson_lookup_utf8 (&args, "to");
+
+   res = mongoc_collection_rename (collection, db, to, true, &error);
+
+   /* This operation is only run by change stream tests, which use
+      it to trigger further events and check all results elsewhere. */
+   ASSERT_OR_PRINT (res, error);
+
+   bson_destroy (&args);
+}
+
+
+static void
+drop (mongoc_collection_t *collection,
+      const bson_t *test,
+      const bson_t *operation,
+      mongoc_client_session_t *session,
+      mongoc_write_concern_t *wc)
+{
+   bson_error_t error;
+   bool res;
+
+   res = mongoc_collection_drop (collection, &error);
+
+   /* This operation is only run by change stream tests, which use
+      it to trigger further events and check all results elsewhere. */
+   ASSERT_OR_PRINT (res, error);
+}
+
+
 static void
 count (mongoc_collection_t *collection,
        const bson_t *test,
@@ -1082,15 +1142,25 @@ distinct (mongoc_collection_t *collection,
           mongoc_client_session_t *session,
           const mongoc_read_prefs_t *read_prefs)
 {
+   bson_t args;
    bson_t opts = BSON_INITIALIZER;
    const char *field_name;
+   bson_t query;
    bson_t reply;
    bson_value_t value = {0};
    bson_error_t error;
    bool r;
 
    append_session (session, &opts);
-   field_name = bson_lookup_utf8 (operation, "arguments.fieldName");
+   bson_lookup_doc (operation, "arguments", &args);
+   field_name = bson_lookup_utf8 (&args, "fieldName");
+   if (bson_has_field (&args, "filter")) {
+      bson_lookup_doc (&args, "filter", &query);
+      BSON_APPEND_DOCUMENT (&opts, "query", &query);
+   }
+
+   COPY_EXCEPT ("fieldName", "filter");
+
    r = mongoc_collection_read_command_with_opts (
       collection,
       tmp_bson (
@@ -1212,6 +1282,26 @@ find (mongoc_collection_t *collection,
 }
 
 
+static bool
+_is_aggregate_out (const bson_t *pipeline)
+{
+   bson_iter_t iter;
+   bson_iter_t stage;
+
+   ASSERT (bson_iter_init (&iter, pipeline));
+   while (bson_iter_next (&iter)) {
+      if (BSON_ITER_HOLDS_DOCUMENT (&iter) &&
+          bson_iter_recurse (&iter, &stage)) {
+         if (bson_iter_find (&stage, "$out")) {
+            return true;
+         }
+      }
+   }
+
+   return false;
+}
+
+
 static void
 aggregate (mongoc_collection_t *collection,
            const bson_t *test,
@@ -1232,7 +1322,28 @@ aggregate (mongoc_collection_t *collection,
    cursor = mongoc_collection_aggregate (
       collection, MONGOC_QUERY_NONE, &pipeline, &opts, read_prefs);
 
-   check_cursor (cursor, test, operation);
+   /* Driver CRUD API Spec: "$out is a special pipeline stage that causes no
+    * results to be returned from the server. As such, the iterable here would
+    * never contain documents. Drivers MAY setup a cursor to be executed upon
+    * iteration against the $out collection such that if a user were to iterate
+    * a pipeline including $out, results would be returned."
+    *
+    * The C Driver chooses the first option, and returns an empty cursor.
+    */
+   if (_is_aggregate_out (&pipeline)) {
+      const bson_t *doc;
+      bson_error_t error;
+      bson_value_t value;
+
+      ASSERT (!mongoc_cursor_next (cursor, &doc));
+      if (mongoc_cursor_error_document (cursor, &error, &doc)) {
+         value_init_from_doc (&value, doc);
+         check_result (test, operation, false, &value, &error);
+      }
+   } else {
+      check_cursor (cursor, test, operation);
+   }
+
    mongoc_cursor_destroy (cursor);
    bson_destroy (&opts);
 }
@@ -1264,7 +1375,7 @@ command (mongoc_database_t *db,
     */
    bson_lookup_doc (operation, "arguments", &args);
    bson_lookup_doc (&args, "command", &cmd);
-   COPY_EXCEPT (&args, &opts, "command");
+   COPY_EXCEPT ("command");
    append_session (session, &opts);
 
    r = mongoc_database_command_with_opts (
@@ -1355,12 +1466,17 @@ json_test_operation (json_test_ctx_t *ctx,
    const char *op_name;
    mongoc_read_prefs_t *read_prefs = NULL;
    mongoc_write_concern_t *wc;
+   mongoc_database_t *db = mongoc_database_copy (ctx->db);
+   mongoc_collection_t *c = mongoc_collection_copy (collection);
 
    op_name = bson_lookup_utf8 (operation, "name");
-   /* databaseOptions don't yet exist in tests, therefore not implemented */
-   BSON_ASSERT (!bson_has_field (operation, "databaseOptions"));
+
+   if (bson_has_field (operation, "databaseOptions")) {
+      bson_lookup_database_opts (operation, "databaseOptions", db);
+   }
+
    if (bson_has_field (operation, "collectionOptions")) {
-      bson_lookup_collection_opts (operation, "collectionOptions", collection);
+      bson_lookup_collection_opts (operation, "collectionOptions", c);
    }
 
    if (bson_has_field (operation, "read_preference")) {
@@ -1379,34 +1495,38 @@ json_test_operation (json_test_ctx_t *ctx,
    }
 
    if (!strcmp (op_name, "bulkWrite")) {
-      bulk_write (collection, test, operation, session, wc);
+      bulk_write (c, test, operation, session, wc);
    } else if (!strcmp (op_name, "deleteOne") ||
               !strcmp (op_name, "deleteMany") ||
               !strcmp (op_name, "insertOne") ||
               !strcmp (op_name, "replaceOne") ||
               !strcmp (op_name, "updateOne") ||
               !strcmp (op_name, "updateMany")) {
-      single_write (collection, test, operation, session, wc);
+      single_write (c, test, operation, session, wc);
    } else if (!strcmp (op_name, "findOneAndDelete") ||
               !strcmp (op_name, "findOneAndReplace") ||
               !strcmp (op_name, "findOneAndUpdate")) {
-      find_and_modify (collection, test, operation, session, wc);
+      find_and_modify (c, test, operation, session, wc);
    } else if (!strcmp (op_name, "insertMany")) {
-      insert_many (collection, test, operation, session, wc);
+      insert_many (c, test, operation, session, wc);
+   } else if (!strcmp (op_name, "rename")) {
+      rename_op (c, test, operation, session, wc);
+   } else if (!strcmp (op_name, "drop")) {
+      drop (c, test, operation, session, wc);
    } else if (!strcmp (op_name, "count")) {
-      count (collection, test, operation, session, read_prefs);
+      count (c, test, operation, session, read_prefs);
    } else if (!strcmp (op_name, "estimatedDocumentCount")) {
-      count (collection, test, operation, session, read_prefs);
+      count (c, test, operation, session, read_prefs);
    } else if (!strcmp (op_name, "countDocuments")) {
-      count (collection, test, operation, session, read_prefs);
+      count (c, test, operation, session, read_prefs);
    } else if (!strcmp (op_name, "distinct")) {
-      distinct (collection, test, operation, session, read_prefs);
+      distinct (c, test, operation, session, read_prefs);
    } else if (!strcmp (op_name, "find")) {
-      find (collection, test, operation, session, read_prefs);
+      find (c, test, operation, session, read_prefs);
    } else if (!strcmp (op_name, "aggregate")) {
-      aggregate (collection, test, operation, session, read_prefs);
+      aggregate (c, test, operation, session, read_prefs);
    } else if (!strcmp (op_name, "runCommand")) {
-      command (ctx->db, test, operation, session, read_prefs);
+      command (db, test, operation, session, read_prefs);
    } else if (!strcmp (op_name, "startTransaction")) {
       start_transaction (ctx, test, operation);
    } else if (!strcmp (op_name, "commitTransaction")) {
@@ -1419,6 +1539,8 @@ json_test_operation (json_test_ctx_t *ctx,
 
    mongoc_read_prefs_destroy (read_prefs);
    mongoc_write_concern_destroy (wc);
+   mongoc_collection_destroy (c);
+   mongoc_database_destroy (db);
 }
 
 
