@@ -560,23 +560,23 @@ request_matches_msg (const request_t *request,
 
       if (i > request->docs.len) {
          fprintf (stderr,
-                  "Expected at least %d documents in request, got %zu\n",
+                  "Expected at least %d documents in request, got %d\n",
                   i,
-                  request->docs.len);
+                  (int) request->docs.len);
          return false;
       }
 
       doc = request_get_doc (request, i);
       /* pass is_command=true for first doc, including "find" command */
       is_command_doc = (i == 0);
-      BSON_ASSERT (match_bson (doc, pattern, is_command_doc));
+      assert_match_bson (doc, pattern, is_command_doc);
    }
 
    if (n_docs < request->docs.len) {
       fprintf (stderr,
-               "Expected %zu documents in request, got %zu\n",
-               n_docs,
-               request->docs.len);
+               "Expected %d documents in request, got %d\n",
+               (int) n_docs,
+               (int) request->docs.len);
       return false;
    }
 
