@@ -220,6 +220,28 @@ bson_lookup_doc_null_ok (const bson_t *b, const char *key, bson_t *doc)
 }
 
 
+/*--------------------------------------------------------------------------
+ *
+ * bson_lookup_bool --
+ *
+ *       Return a bool by key, or BSON_ASSERT and abort.
+ *
+ *--------------------------------------------------------------------------
+ */
+bool
+bson_lookup_bool (const bson_t *b, const char *key)
+{
+   bson_iter_t iter;
+   bson_iter_t descendent;
+
+   bson_iter_init (&iter, b);
+   BSON_ASSERT (bson_iter_find_descendant (&iter, key, &descendent));
+   BSON_ASSERT (BSON_ITER_HOLDS_BOOL (&descendent));
+
+   return bson_iter_bool (&descendent);
+}
+
+
 /*--------------------------------------------------------------------------
  *
  * bson_lookup_int32 --
@@ -306,8 +328,9 @@ bson_lookup_write_concern (const bson_t *b, const char *key)
    bson_lookup_doc (b, key, &doc);
    BSON_ASSERT (bson_iter_init (&iter, &doc));
 
-   /* current command monitoring tests always have "w" and no other fields */
-   ASSERT_CMPUINT32 (bson_count_keys (&doc), ==, (uint32_t) 1);
+   /* current command monitoring tests always have "w" but may also have
+    * "wtimeout" and "j" fields. */
+   ASSERT_CMPUINT32 (bson_count_keys (&doc), >=, (uint32_t) 1);
    BSON_ASSERT (bson_iter_find_descendant (&iter, "w", &w));
 
    if (BSON_ITER_HOLDS_NUMBER (&w)) {
@@ -318,6 +341,14 @@ bson_lookup_write_concern (const bson_t *b, const char *key)
       mongoc_write_concern_set_wtag (wc, bson_iter_utf8 (&w, NULL));
    }
 
+   if (bson_has_field (&doc, "wtimeout")) {
+      mongoc_write_concern_set_wtimeout (wc, bson_lookup_int32 (&doc, "wtimeout"));
+   }
+
+   if (bson_has_field (&doc, "j")) {
+      mongoc_write_concern_set_journal (wc, bson_lookup_bool (&doc, "j"));
+   }
+
    return wc;
 }
 
@@ -360,6 +391,46 @@ bson_lookup_read_prefs (const bson_t *b, const char *key)
 }
 
 
+/*--------------------------------------------------------------------------
+ *
+ * bson_lookup_database_opts --
+ *
+ *       Interpret a subdocument as database options.
+ *
+ *--------------------------------------------------------------------------
+ */
+void
+bson_lookup_database_opts (const bson_t *b,
+                           const char *key,
+                           mongoc_database_t *database)
+{
+   bson_t doc;
+   mongoc_read_concern_t *rc;
+   mongoc_write_concern_t *wc;
+   mongoc_read_prefs_t *prefs;
+
+   bson_lookup_doc (b, key, &doc);
+
+   if (bson_has_field (&doc, "readConcern")) {
+      rc = bson_lookup_read_concern (&doc, "readConcern");
+      mongoc_database_set_read_concern (database, rc);
+      mongoc_read_concern_destroy (rc);
+   }
+
+   if (bson_has_field (&doc, "writeConcern")) {
+      wc = bson_lookup_write_concern (&doc, "writeConcern");
+      mongoc_database_set_write_concern (database, wc);
+      mongoc_write_concern_destroy (wc);
+   }
+
+   if (bson_has_field (&doc, "readPreference")) {
+      prefs = bson_lookup_read_prefs (&doc, "readPreference");
+      mongoc_database_set_read_prefs (database, prefs);
+      mongoc_read_prefs_destroy (prefs);
+   }
+}
+
+
 /*--------------------------------------------------------------------------
  *
  * bson_lookup_collection_opts --
@@ -507,7 +578,7 @@ static bool
 is_empty_doc_or_array (const bson_value_t *value);
 
 static bool
-find (bson_value_t *value,
+find (bson_iter_t *iter,
       const bson_t *doc,
       const char *key,
       bool is_command,
@@ -559,7 +630,7 @@ single_quotes_to_double (const char *str)
  *       True or false.
  *
  * Side effects:
- *       Logs if no match.
+ *       Logs if no match. Aborts if json is malformed.
  *
  *--------------------------------------------------------------------------
  */
@@ -579,8 +650,7 @@ match_json (const bson_t *doc,
    char *double_quoted;
    bson_error_t error;
    bson_t *pattern;
-   char errmsg[1000];
-   match_ctx_t ctx = {0};
+   match_ctx_t ctx = {{0}};
    bool matches;
 
    va_start (args, json_pattern);
@@ -596,23 +666,25 @@ match_json (const bson_t *doc,
       abort ();
    }
 
-   ctx.errmsg = errmsg;
-   ctx.errmsg_len = sizeof errmsg;
-   matches = match_bson_with_ctx (doc, pattern, is_command, &ctx);
+   ctx.is_command = is_command;
+   matches = match_bson_with_ctx (doc, pattern, &ctx);
 
    if (!matches) {
+      char *as_string =
+         doc ? bson_as_canonical_extended_json (doc, NULL) : NULL;
       fprintf (stderr,
                "ASSERT_MATCH failed with document:\n\n"
                "%s\n"
                "pattern:\n%s\n"
                "%s\n"
                "%s:%d %s()\n",
-               doc ? bson_as_canonical_extended_json (doc, NULL) : "{}",
+               as_string ? as_string : "{}",
                double_quoted,
                ctx.errmsg,
                filename,
                lineno,
                funcname);
+      bson_free (as_string);
    }
 
    bson_destroy (pattern);
@@ -643,15 +715,49 @@ match_json (const bson_t *doc,
 bool
 match_bson (const bson_t *doc, const bson_t *pattern, bool is_command)
 {
-   match_ctx_t ctx = {0};
+   match_ctx_t ctx = {{0}};
 
    ctx.strict_numeric_types = true;
-   return match_bson_with_ctx (doc, pattern, is_command, &ctx);
+   ctx.is_command = is_command;
+
+   return match_bson_with_ctx (doc, pattern, &ctx);
+}
+
+/*
+ *--------------------------------------------------------------------------
+ *
+ * assert_match_bson --
+ *
+ *       Test helper that wraps match_bson. Fails the test if there
+ *       is no found match.
+ *
+ * Returns:
+ *       Nothing.
+ *
+ * Side effects:
+ *       abort()s if there is no match.
+ *
+ *--------------------------------------------------------------------------
+ */
+void
+assert_match_bson (const bson_t *doc, const bson_t *pattern, bool is_command)
+{
+   match_ctx_t ctx = {{0}};
+
+   ctx.strict_numeric_types = true;
+   ctx.is_command = is_command;
+
+   if (!match_bson_with_ctx (doc, pattern, &ctx)) {
+      test_error ("Expected: %s\n, Got: %s\n, %s\n",
+                  bson_as_canonical_extended_json (pattern, NULL),
+                  bson_as_canonical_extended_json (doc, NULL),
+                  ctx.errmsg);
+   }
 }
 
 
 MONGOC_PRINTF_FORMAT (2, 3)
-static void
+void
 match_err (match_ctx_t *ctx, const char *fmt, ...)
 {
    va_list args;
@@ -663,12 +769,8 @@ match_err (match_ctx_t *ctx, const char *fmt, ...)
    formatted = bson_strdupv_printf (fmt, args);
    va_end (args);
 
-   if (ctx->errmsg) {
-      bson_snprintf (
-         ctx->errmsg, ctx->errmsg_len, "%s: %s", ctx->path, formatted);
-   } else {
-      test_error ("%s: %s", ctx->path, formatted);
-   }
+   bson_snprintf (
+      ctx->errmsg, sizeof ctx->errmsg, "%s: %s", ctx->path, formatted);
 
    bson_free (formatted);
 }
@@ -684,8 +786,6 @@ derive (match_ctx_t *ctx, match_ctx_t *derived, const char *key)
    BSON_ASSERT (key);
 
    derived->strict_numeric_types = ctx->strict_numeric_types;
-   derived->errmsg = ctx->errmsg; /* location to write err msg if any */
-   derived->errmsg_len = ctx->errmsg_len;
 
    if (strlen (ctx->path) > 0) {
       bson_snprintf (
@@ -695,6 +795,10 @@ derive (match_ctx_t *ctx, match_ctx_t *derived, const char *key)
    }
    derived->retain_dots_in_keys = ctx->retain_dots_in_keys;
    derived->allow_placeholders = ctx->allow_placeholders;
+   derived->visitor_ctx = ctx->visitor_ctx;
+   derived->visitor_fn = ctx->visitor_fn;
+   derived->is_command = false;
+   derived->errmsg[0] = 0;
 }
 
 
@@ -713,7 +817,10 @@ derive (match_ctx_t *ctx, match_ctx_t *derived, const char *key)
  *       The only special pattern syntaxes are "field": {"$exists": true/false}
  *       and "field": {"$empty": true/false}.
  *
- *       The first key matches case-insensitively if is_command.
+ *       The first key matches case-insensitively if ctx->is_command.
+ *
+ *       An optional match visitor (match_visitor_fn and match_visitor_ctx)
+ *       can be set in ctx to provide custom matching behavior.
  *
  *       A NULL doc or NULL pattern means "{}".
  *
@@ -727,10 +834,7 @@ derive (match_ctx_t *ctx, match_ctx_t *derived, const char *key)
  */
 
 bool
-match_bson_with_ctx (const bson_t *doc,
-                     const bson_t *pattern,
-                     bool is_command,
-                     match_ctx_t *ctx)
+match_bson_with_ctx (const bson_t *doc, const bson_t *pattern, match_ctx_t *ctx)
 {
    bson_iter_t pattern_iter;
    const char *key;
@@ -740,7 +844,8 @@ match_bson_with_ctx (const bson_t *doc,
    bool is_empty_operator;
    bool exists;
    bool empty;
-   bool found = false;
+   bool found;
+   bson_iter_t doc_iter;
    bson_value_t doc_value;
    match_ctx_t derived;
 
@@ -760,8 +865,15 @@ match_bson_with_ctx (const bson_t *doc,
    while (bson_iter_next (&pattern_iter)) {
       key = bson_iter_key (&pattern_iter);
       value = bson_iter_value (&pattern_iter);
-      found = find (
-         &doc_value, doc, key, is_command, is_first, ctx->retain_dots_in_keys);
+      found = find (&doc_iter,
+                    doc,
+                    key,
+                    ctx->is_command,
+                    is_first,
+                    ctx->retain_dots_in_keys);
+      if (found) {
+         bson_value_copy (bson_iter_value (&doc_iter), &doc_value);
+      }
 
       /* is value {"$exists": true} or {"$exists": false} ? */
       is_exists_operator = get_exists_operator (value, &exists);
@@ -771,7 +883,23 @@ match_bson_with_ctx (const bson_t *doc,
 
       derive (ctx, &derived, key);
 
-      if (is_exists_operator) {
+      if (ctx->visitor_fn) {
+         match_action_t action =
+            ctx->visitor_fn (ctx, &pattern_iter, found ? &doc_iter : NULL);
+         if (action == MATCH_ACTION_ABORT) {
+            goto fail;
+         } else if (action == MATCH_ACTION_SKIP) {
+            goto next;
+         }
+      }
+
+      if (value->value_type == BSON_TYPE_NULL && found) {
+         /* pattern has "key": null, and "key" is in doc */
+         if (doc_value.value_type != BSON_TYPE_NULL) {
+            match_err (&derived, "%s should be null or absent", key);
+            goto fail;
+         }
+      } else if (is_exists_operator) {
          if (exists != found) {
             match_err (&derived, "%s found", found ? "" : "not");
             goto fail;
@@ -788,6 +916,7 @@ match_bson_with_ctx (const bson_t *doc,
          goto fail;
       }
 
+   next:
       is_first = false;
       if (found) {
          bson_value_destroy (&doc_value);
@@ -801,6 +930,10 @@ fail:
       bson_value_destroy (&doc_value);
    }
 
+   if (strlen (derived.errmsg) > 0) {
+      strcpy (ctx->errmsg, derived.errmsg);
+   }
+
    return false;
 }
 
@@ -815,13 +948,13 @@ fail:
  *       Whether the key was found.
  *
  * Side effects:
- *       Copies the found value into "value".
+ *       Copies the found value into "iter_out".
  *
  *--------------------------------------------------------------------------
  */
 
 static bool
-find (bson_value_t *value,
+find (bson_iter_t *iter_out,
       const bson_t *doc,
       const char *key,
       bool is_command,
@@ -838,7 +971,7 @@ find (bson_value_t *value,
          return false;
       }
 
-      bson_value_copy (bson_iter_value (&descendent), value);
+      memcpy (iter_out, &descendent, sizeof (bson_iter_t));
       return true;
    } else if (is_command && is_first) {
       if (!bson_iter_find_case (&iter, key)) {
@@ -848,7 +981,7 @@ find (bson_value_t *value,
       return false;
    }
 
-   bson_value_copy (bson_iter_value (&iter), value);
+   memcpy (iter_out, &iter, sizeof (bson_iter_t));
    return true;
 }
 
@@ -1097,7 +1230,7 @@ match_bson_value (const bson_value_t *doc,
       if (doc->value_type == BSON_TYPE_ARRAY) {
          ret = match_bson_arrays (&subdoc, &pattern_subdoc, ctx);
       } else {
-         ret = match_bson_with_ctx (&subdoc, &pattern_subdoc, false, ctx);
+         ret = match_bson_with_ctx (&subdoc, &pattern_subdoc, ctx);
       }
 
       bson_destroy (&subdoc);
@@ -1400,7 +1533,7 @@ match_in_array (const bson_t *doc, const bson_t *array, match_ctx_t *ctx)
       ASSERT (BSON_ITER_HOLDS_DOCUMENT (&array_iter));
       bson_iter_bson (&array_iter, &array_elem);
 
-      if (match_bson_with_ctx (&array_elem, doc, false, ctx)) {
+      if (match_bson_with_ctx (&array_elem, doc, ctx)) {
          found = true;
       }
 
@@ -1537,4 +1670,4 @@ json_with_all_types ()
                       "    }\n"
                       "}";
    return json;
-}
\ No newline at end of file
+}
