@@ -58,8 +58,6 @@ test_endian_install (TestSuite *suite);
 extern void
 test_bson_error_install (TestSuite *suite);
 extern void
-test_fnv_install (TestSuite *suite);
-extern void
 test_iso8601_install (TestSuite *suite);
 extern void
 test_iter_install (TestSuite *suite);
@@ -121,6 +119,8 @@ test_exhaust_install (TestSuite *suite);
 extern void
 test_find_and_modify_install (TestSuite *suite);
 extern void
+test_gridfs_bucket_install (TestSuite *suite);
+extern void
 test_gridfs_file_page_install (TestSuite *suite);
 extern void
 test_gridfs_install (TestSuite *suite);
@@ -210,6 +210,8 @@ extern void
 test_counters_install (TestSuite *suite);
 extern void
 test_crud_install (TestSuite *suite);
+extern void
+test_apm_install (TestSuite *suite);
 
 typedef struct {
    mongoc_log_level_t level;
@@ -821,7 +823,7 @@ test_framework_get_user_password (char **user, char **password)
 #ifndef MONGOC_ENABLE_CRYPTO
    if (*user && *password) {
       fprintf (stderr,
-               "You need to configure with --enable-ssl"
+               "You need to configure with ENABLE_SSL"
                " when providing user+password (for SCRAM-SHA-1)\n");
       abort ();
    }
@@ -939,43 +941,6 @@ test_framework_has_compressors ()
    return retval;
 }
 
-
-/*
- *--------------------------------------------------------------------------
- *
- * test_framework_add_compressors_from_env --
- *
- *       Add supported compressors to the URI
- *
- * Returns:
- *       A string you must bson_free.
- *
- *--------------------------------------------------------------------------
- */
-char *
-test_framework_add_compressors_from_env (const char *uri_str)
-{
-   char *compressors;
-
-   compressors = test_framework_get_compressors ();
-   if (compressors) {
-      char *retval = NULL;
-
-      if (!strstr ("?", uri_str)) {
-         retval = bson_strdup_printf (
-            "%s&%s=%s", uri_str, MONGOC_URI_COMPRESSORS, compressors);
-      } else {
-         retval = bson_strdup_printf (
-            "%s/?%s=%s", uri_str, MONGOC_URI_COMPRESSORS, compressors);
-      }
-      bson_free (compressors);
-      return retval;
-   }
-
-   return bson_strdup (uri_str);
-}
-
-
 /*
  *--------------------------------------------------------------------------
  *
@@ -1317,15 +1282,14 @@ char *
 test_framework_get_uri_str ()
 {
    char *uri_str_no_auth;
-   char *uri_str_auth;
    char *uri_str;
 
+   /* no_auth also contains compressors. */
+
    uri_str_no_auth = test_framework_get_uri_str_no_auth (NULL);
-   uri_str_auth = test_framework_add_user_password_from_env (uri_str_no_auth);
-   uri_str = test_framework_add_compressors_from_env (uri_str_auth);
+   uri_str = test_framework_add_user_password_from_env (uri_str_no_auth);
 
    bson_free (uri_str_no_auth);
-   bson_free (uri_str_auth);
 
    return uri_str;
 }
@@ -2204,6 +2168,7 @@ WIRE_VERSION_CHECKS (4)
 WIRE_VERSION_CHECKS (5)
 WIRE_VERSION_CHECKS (6)
 WIRE_VERSION_CHECKS (7)
+WIRE_VERSION_CHECKS (8)
 
 int
 test_framework_skip_if_no_dual_ip_hostname (void)
@@ -2306,7 +2271,6 @@ main (int argc, char *argv[])
    test_clock_install (&suite);
    test_decimal128_install (&suite);
    test_endian_install (&suite);
-   test_fnv_install (&suite);
    test_iso8601_install (&suite);
    test_iter_install (&suite);
    test_json_install (&suite);
@@ -2340,6 +2304,7 @@ main (int argc, char *argv[])
    test_exhaust_install (&suite);
    test_find_and_modify_install (&suite);
    test_gridfs_install (&suite);
+   test_gridfs_bucket_install (&suite);
    test_gridfs_file_page_install (&suite);
    test_handshake_install (&suite);
    test_linux_distro_scanner_install (&suite);
@@ -2386,6 +2351,7 @@ main (int argc, char *argv[])
    test_happy_eyeballs_install (&suite);
    test_counters_install (&suite);
    test_crud_install (&suite);
+   test_apm_install (&suite);
 
    ret = TestSuite_Run (&suite);
 
