@@ -52,20 +52,15 @@ test_session_no_crypto (void *ctx)
 }
 
 
-#define ASSERT_SESSIONS_MATCH(_lsid_a, _lsid_b)               \
-   do {                                                       \
-      BSON_ASSERT (match_bson ((_lsid_a), (_lsid_b), false)); \
+#define ASSERT_SESSIONS_MATCH(_lsid_a, _lsid_b)        \
+   do {                                                \
+      assert_match_bson ((_lsid_a), (_lsid_b), false); \
    } while (0)
 
 
-#define ASSERT_SESSIONS_DIFFER(_lsid_a, _lsid_b)                               \
-   do {                                                                        \
-      /* need a match context when checking that lsids DON'T match */          \
-      char errmsg[1000];                                                       \
-      match_ctx_t _ctx = {0};                                                  \
-      _ctx.errmsg = errmsg;                                                    \
-      _ctx.errmsg_len = sizeof (errmsg);                                       \
-      BSON_ASSERT (!match_bson_with_ctx ((_lsid_a), (_lsid_b), false, &_ctx)); \
+#define ASSERT_SESSIONS_DIFFER(_lsid_a, _lsid_b)               \
+   do {                                                        \
+      BSON_ASSERT (!match_bson ((_lsid_a), (_lsid_b), false)); \
    } while (0)
 
 
@@ -625,8 +620,7 @@ _test_end_sessions (bool pooled)
    bson_t ended_lsids;
    bson_iter_t iter;
    bson_t ended_lsid;
-   char errmsg[1000];
-   match_ctx_t ctx = {0};
+   match_ctx_t ctx = {{0}};
    bool r;
 
    endsessions_test_init (&test, pooled);
@@ -668,16 +662,13 @@ _test_end_sessions (bool pooled)
 
    endsessions_test_get_ended_lsids (&test, 0, &ended_lsids);
 
-   ctx.errmsg = errmsg;
-   ctx.errmsg_len = sizeof errmsg;
-
    BSON_ASSERT (bson_iter_init (&iter, &ended_lsids));
    while (bson_iter_next (&iter)) {
       BSON_ASSERT (BSON_ITER_HOLDS_DOCUMENT (&iter));
       bson_iter_bson (&iter, &ended_lsid);
-      if (match_bson_with_ctx (&ended_lsid, &lsid1, false, &ctx)) {
+      if (match_bson_with_ctx (&ended_lsid, &lsid1, &ctx)) {
          lsid1_ended = true;
-      } else if (match_bson_with_ctx (&ended_lsid, &lsid2, false, &ctx)) {
+      } else if (match_bson_with_ctx (&ended_lsid, &lsid2, &ctx)) {
          lsid2_ended = true;
       }
    }
@@ -781,11 +772,11 @@ _test_advance_cluster_time (mongoc_client_session_t *cs,
    mongoc_client_session_advance_cluster_time (cs, new_cluster_time);
 
    if (should_advance) {
-      BSON_ASSERT (match_bson (
-         mongoc_client_session_get_cluster_time (cs), new_cluster_time, false));
+      assert_match_bson (
+         mongoc_client_session_get_cluster_time (cs), new_cluster_time, false);
    } else {
-      BSON_ASSERT (match_bson (
-         mongoc_client_session_get_cluster_time (cs), old_cluster_time, false));
+      assert_match_bson (
+         mongoc_client_session_get_cluster_time (cs), old_cluster_time, false);
    }
 
    bson_destroy (old_cluster_time);
@@ -922,8 +913,7 @@ typedef struct {
 static void
 started (const mongoc_apm_command_started_t *event)
 {
-   char errmsg[1000];
-   match_ctx_t ctx = {errmsg, sizeof (errmsg), false /* strict numbers */};
+   match_ctx_t ctx = {{0}};
    bson_iter_t iter;
    bool has_cluster_time;
    bson_t cluster_time;
@@ -934,6 +924,8 @@ started (const mongoc_apm_command_started_t *event)
    session_test_t *test =
       (session_test_t *) mongoc_apm_command_started_get_context (event);
 
+   ctx.strict_numeric_types = false;
+
    if (test->verbose) {
       char *s = bson_as_json (cmd, NULL);
       printf ("%s\n", s);
@@ -956,14 +948,14 @@ started (const mongoc_apm_command_started_t *event)
       client_session_lsid = &test->cs->server_session->lsid;
 
       if (test->expect_explicit_lsid) {
-         if (!match_bson_with_ctx (&lsid, client_session_lsid, false, &ctx)) {
+         if (!match_bson_with_ctx (&lsid, client_session_lsid, &ctx)) {
             fprintf (stderr,
                      "command %s should have used client session's lsid\n",
                      cmd_name);
             abort ();
          }
       } else {
-         if (match_bson_with_ctx (&lsid, client_session_lsid, false, &ctx)) {
+         if (match_bson_with_ctx (&lsid, client_session_lsid, &ctx)) {
             fprintf (stderr,
                      "command %s should not have used client session's lsid\n",
                      cmd_name);
@@ -975,7 +967,7 @@ started (const mongoc_apm_command_started_t *event)
          bson_destroy (&test->sent_lsid);
          bson_copy_to (&lsid, &test->sent_lsid);
       } else {
-         if (!match_bson_with_ctx (&lsid, &test->sent_lsid, false, &ctx)) {
+         if (!match_bson_with_ctx (&lsid, &test->sent_lsid, &ctx)) {
             fprintf (stderr,
                      "command %s used different lsid than previous command\n",
                      cmd_name);
@@ -1144,15 +1136,16 @@ session_test_new (session_test_correct_t correct_client,
 static void
 check_session_returned (session_test_t *test, const bson_t *lsid)
 {
-   char errmsg[1000];
-   match_ctx_t ctx = {errmsg, sizeof (errmsg), false /* strict numbers */};
+   match_ctx_t ctx = {{0}};
    mongoc_server_session_t *ss;
    bool found;
 
+   ctx.strict_numeric_types = false;
+
    found = false;
    CDL_FOREACH (test->session_client->topology->session_pool, ss)
    {
-      if (match_bson_with_ctx (&ss->lsid, lsid, false, &ctx)) {
+      if (match_bson_with_ctx (&ss->lsid, lsid, &ctx)) {
          found = true;
          break;
       }
@@ -2050,7 +2043,6 @@ _test_bulk (session_test_t *test, mongoc_bulk_operation_t *bulk)
 }
 
 
-
 /* test the standard mongoc_collection_create_bulk_operation_with_opts */
 static void
 test_bulk (session_test_t *test)
@@ -2641,12 +2633,12 @@ test_session_install (TestSuite *suite)
       suite, "/Session/collection_names", test_collection_names, true);
    add_session_test (suite, "/Session/bulk", test_bulk, false);
    add_session_test (suite, "/Session/find_indexes", test_find_indexes, true);
-   TestSuite_AddFull (suite,                                          
+   TestSuite_AddFull (suite,
                       "/Session/bulk_set_session",
                       run_session_test_bulk_operation,
-                      NULL,                                            
+                      NULL,
                       test_bulk_set_session,
-                      test_framework_skip_if_no_cluster_time,          
+                      test_framework_skip_if_no_cluster_time,
                       test_framework_skip_if_no_crypto);
    TestSuite_AddFull (suite,
                       "/Session/bulk_set_client",
