@@ -1,6 +1,6 @@
-#include <mongoc.h>
-#include <mongoc-cursor-private.h>
-#include <mongoc-client-private.h>
+#include <mongoc/mongoc.h>
+#include <mongoc/mongoc-cursor-private.h>
+#include <mongoc/mongoc-client-private.h>
 
 #include "TestSuite.h"
 #include "test-libmongoc.h"
@@ -57,7 +57,7 @@ _insert_test_docs (mongoc_collection_t *collection, const bson_t *docs)
    bson_iter_init (&iter, docs);
    while (bson_iter_next (&iter)) {
       bson_iter_document (&iter, &len, &data);
-      bson_init_static (&doc, data, len);
+      BSON_ASSERT (bson_init_static (&doc, data, len));
       r = mongoc_collection_insert_one (collection, &doc, NULL, NULL, &error);
       ASSERT_OR_PRINT (r, error);
    }
@@ -288,7 +288,7 @@ _reply_to_op_query (request_t *request, test_collection_find_t *test_data)
    for (i = 0; i < test_data->n_results; i++) {
       bson_iter_next (&iter);
       bson_iter_document (&iter, &len, &data);
-      bson_init_static (&docs[i], data, len);
+      BSON_ASSERT (bson_init_static (&docs[i], data, len));
    }
 
    mock_server_reply_multi (request,
