@@ -1,13 +1,13 @@
-#include <mongoc.h>
-#include <mongoc-collection-private.h>
-#include <mongoc-write-concern-private.h>
-#include <mongoc-read-concern-private.h>
-#include <mongoc-util-private.h>
+#include <mongoc/mongoc.h>
+#include <mongoc/mongoc-collection-private.h>
+#include <mongoc/mongoc-write-concern-private.h>
+#include <mongoc/mongoc-read-concern-private.h>
+#include <mongoc/mongoc-util-private.h>
 
 #include "TestSuite.h"
 #include "test-libmongoc.h"
-#include "mongoc-client-private.h"
-#include "mongoc-database-private.h"
+#include "mongoc/mongoc-client-private.h"
+#include "mongoc/mongoc-database-private.h"
 #include "mock_server/future-functions.h"
 #include "mock_server/mock-server.h"
 #include "test-conveniences.h"
