@@ -11,8 +11,7 @@
 static void
 _assert_options_match (const bson_t *test, mongoc_client_t *client)
 {
-   char errmsg[1000];
-   match_ctx_t ctx = {0};
+   match_ctx_t ctx = {{0}};
    bson_iter_t iter;
    bson_t opts_from_test;
    const bson_t *opts_from_uri;
@@ -23,9 +22,6 @@ _assert_options_match (const bson_t *test, mongoc_client_t *client)
    const char *opt_name;
    const bson_value_t *test_value, *uri_value;
 
-   ctx.errmsg = errmsg;
-   ctx.errmsg_len = sizeof errmsg;
-
    if (!bson_iter_init_find (&iter, test, "options")) {
       /* no URI options specified in the test */
       return;
